# 2014. 12. 9 by Hans Roh hansroh@gmail.com

VERSION = "0.12"
version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  VERSION.split (".")))

from .analyzers import standardAnalyzer
from .searcher import collection
from .classifier import model
from . import memory
from . import analyzers
import platform
import os
from .searcher import indexer, document
from .classifier import trainer, labeld_document

PID = os.getpid ()

standard_analyzer = standardAnalyzer.Analyzer

collection = collection.Collection
model = model.Model

document = document.Document
labeled_document = labeld_document.LabeledDocument

TEXT = "Text"
TERM = "Term"
STRING = "String"
LIST = "List"
COORD = "Coord4" 
COORD4 = "Coord4" # 4 decimals, 10 M precision
COORD6 = "Coord6" # 6 decimals, 10 CM precision
COORD8 = "Coord8" # 8 decimals, 1 MM precision
BIT8  = "Bit8"
BIT16 = "Bit16"
BIT24 = "Bit24"
BIT32 = "Bit32"
BIT40 = "Bit40"
BIT48 = "Bit48"
BIT56 = "Bit56"
BIT64 = "Bit64"
INT8  = "Int8"
INT16 = "Int16"
INT24 = "Int24"
INT32 = "Int32"
INT40 = "Int40"
INT48 = "Int48"
INT56 = "Int56"
INT64 = "Int64"

ALL = "default"
NAIVEBAYES = "naivebayes"
FEATUREVOTE = "featurevote"
TFIDF = "tfidf"
SIMILARITY = "similarity"
ROCCHIO = "rocchino"
ROCCHINO = "rocchino"
MULTIPATH = "multipath"
META = "metaclassifier"

CHI2 = "chi"
GSS = "gss"
DF = "df"
NGL = "ngl"
MI = "mi"
TFIDF = "tfidf"
IG = "ig"
OR = "or"
OR4P = "or4p"
RS = "rs"
LOR = "lor"
COS = "cos"
PPHI = "pphi"
YULE = "yule"
RMI = "rmi"

R = "r"

SUM = "sum"
MAX = "max"
AVG = "avg"
MIN = "min"

WRITE = "w"
APPEND = "w"
MODIFY = "w"
READ = "r"
CREATE = "c"

osbit, __x = platform.architecture()
if osbit == "64bit":
	LIMIT_SEGMENTSIZE = 10000000000	# 2**39, 10GB but max 500GB is possible
else:
	LIMIT_SEGMENTSIZE = 1900000000 # 2**31, 2GB
LIMIT_FILESIZE = int (LIMIT_SEGMENTSIZE * 0.7)

logger = None
	
class Task:
	def __init__ (self, logger):
		self.logger = logger
		self._d_ = {}
		
	def cleanup (self):
		for tname, reactor in self._d_.items ():
			self.logger ("... wissen closing %s" % tname, "info")
			reactor.close ()		
			
	def __contains__ (self, tname):
		return (tname in self._d_)
	
	def __getattr__ (self, tname):
		return self._d_ [tname]
	
	def __delattr__ (self, tname):
		self.rempove (tname)

	def __dir__ (self):
		return list (self._d_.keys ())
	
	def __len__ (self):
		return len (self._d_)
		
	def assign (self, tname, obj):
		obj.si.set_ident (tname)
		self._d_ [tname] = obj
	
	def resign (self, tname):
		del self._d_ [tname]
		
	def get (self, tname):
		return self._d_ [tname]
	
	def status (self):
		d = {}
		for tname, reactor in self._d_.items ():
			d [tname] = reactor.status ()
		return d
		

class NoTask:
	def __getattr__ (self, name):
		raise AssertionError ("wissen not configured, configure first")

task = NoTask ()

def configure (numthread, logger_t, io_buf_size = 4096, mem_limit = 128, max_segment_size = 0):
	global logger, task
	
	if isinstance (task, Task):
		return
		
	logger = logger_t
	if max_segment_size:
		set_max_segment_size (max_segment_size)
		
	if not memory.isInitialized ():
		memory.initialize (numthread, io_buf_size, mem_limit, "segment", logger_t)
		analyzers.buildFactory (numthread, logger_t)
	
	task = Task (logger)
	return task
	
def shutdown ():
	global task
	
	if isinstance (task, Task):
		task.cleanup ()
		
	if memory.isInitialized ():
		analyzers.close ()
		memory.destroy ()

# For Skitai
cleanup = shutdown

def set_max_segment_size (mbytes):
	global LIMIT_SEGMENTSIZE, LIMIT_FILESIZE
	LIMIT_SEGMENTSIZE = int (mbytes * 1000000)
	LIMIT_FILESIZE = int (LIMIT_SEGMENTSIZE * 0.7)

def qualify_analyzer (analyzer):
	if memory.isInitialized ():
		return analyzers.checkIn (analyzer) 
	return analyzer	


#=========================================
# query and guess by task alias
#=========================================

#-----------------------------------------
# common jobs
#-----------------------------------------

def assign (tname, obj):
	global task
	task.assign (tname, obj)
	
def resign (tname):
	global task
	task.resign (tname, obj)
		
def close (tname, *args, **karg):
	global task	
	reactor = task.get (tname)
	if not reactor.closed:
		reactor.close (*args, **karg)
	task.remove (tname)

def get (tname):
	global task	
	return task.get (tname)

def status (tname = "", *args, **karg):
	global task
	if tname == "":
		return task.status ()
	return task.get (tname).status (*args, **karg)

def refresh (tname, *args, **karg):
	global task
	return task.get (tname).refresh (*args, **karg)	

#-----------------------------------------
# searcher only jobs	
#-----------------------------------------
def query (tname, *args, **karg):
	global task
	return task.get (tname).query (*args, **karg)

def delete (tname, *args, **karg):
	global task
	return task.get (tname).delete (*args, **karg)		

def remove (tname, *args, **karg):
	global task
	return task.get (tname).remove (*args, **karg)
	
def fetch (tname, *args, **karg):
	global task
	return task.get (tname).fetch (*args, **karg)	

#-----------------------------------------
# classifier only jobs	
#-----------------------------------------
def guess (tname, *args, **karg):
	global task
	return task.get (tname).guess (*args, **karg)	

	
	


from wissen import _wissen
import os

class BinFile:
	def __init__ (self, path, mode = "r"):
		self.path = path
		self.mode = mode		
		
		if mode == "r":
			flag = os.O_RDONLY
		else:
			flag = os.O_WRONLY | os.O_CREAT			
		if os.name == "nt":
			flag |= os.O_BINARY
			
		self.fdno = os.open (self.path, flag)
		self.bf = _wissen.BinFile (self.fdno, self.mode)
		
	def __getattr__ (self, attr):
		return getattr (self.bf, attr)
		
	def close (self):
		self.bf.close ()
		os.close (self.fdno)


def open (path, mode = "r"):
	return BinFile (path, mode)
	


import types
from wissen.lib import unistr

class Document:
	def __init__ (self, content = None):
		self.fields = {}
		self.data = None
		self.stored = []
		self.summary = None		
		if content:
			self.set_content (content)
	
	def set_content (self, content):
		# any encoding even allowed python objects
		if self.stored: raise ValueError("Already got content(s)")
		self.data = content
	
	def add_content (self, content, option = []):
		# any encoding even allowed python objects
		if self.data: 
			self.stored.append (self.data)
			self.data = None
		self.stored.append ((content, option))
			
	def add_field (self, name, value, ftype = "Text", lang = "un", encoding = None, option = {}):
		self.fields [name] = (unistr.makes (value, encoding), ftype, lang, option)
	
	def set_auto_snippet (self, text, option = {}, encoding = None):
		self.summary = (unistr.makes (text, encoding), option)




import re
from . import typeinfo
import wissen

class FieldWriter:
	def __init__ (self, segment, analyzer = None):
		self.segment = segment
		self.analyzer = analyzer
		self.tcache = {}

	#---------------------------------------------------------------------------
	# add posting
	#---------------------------------------------------------------------------
	def indexText (self, type, docid, num, name, value, lang, option):
		if self.analyzer:
			if "removehtml"in option:
				temp = value
				value = self.analyzer.htmlStrip (temp)
			if "formalize" in option:
				value = self.analyzer.formalize (value)					
			if "transform" in option:
				value = self.analyzer.transform (value)
			term = value	
			value = self.analyzer.analyze (value, lang, type)
		
		if not value:
			self.segment.writeNorm (num, 0)
			return
		
		numTerm = 0
		if not value:
			self.segment.writeNorm (num, 0)
			return
		
		if type == wissen.TEXT:
			for term in value:				
				self.segment.writePosting (docid, num, term, len (value [term]), value [term])
				numTerm += len (value [term])

		elif type == wissen.TERM:
			for term, tf in value:
				self.segment.writePosting (docid, num, term, tf)
				numTerm += tf

		self.segment.writeNorm (num, numTerm)

	def indexField (self, type, docid, num, name, value, lang, option):
		if not value: return
		self.segment.writePosting (docid, num, value, 1)

	def indexList (self, ftype, docid, num, name, value, lang, option):
		if not value: return
		if type (value)	is type (""):			
			delim = ","
			if "delim" in option:
				ii = option.index ("delim")
				try:
					delim = option [ii + 1]
				except IndexError:
					pass
			value = value.split (delim)
		
		d = {}
		for each in [_f for _f in [x.strip () for x in value] if _f]:
			d [each] = None
		
		for each in d:
			self.indexField (ftype, docid, num, name, each, lang, option)

	def indexInt (self, type, docid, num, name, value, lang, option):
		try: val = int (value)
		except: val = 0
		self.segment.writeSortKey (num, val, typeinfo.typemap.getsize (type))

	def indexBit (self, type, docid, num, name, value, lang, option):
		try: val = int (value, 2)
		except: val = 0
		self.segment.writeSortKey (num, val, typeinfo.typemap.getsize (type))

	def indexStringSort (self, type, docid, num, name, value, option):
		self.segment.writeStringSortKey (num, value, typeinfo.typemap.getsize (type))

	def indexCoord (self, type, docid, num, name, value, lang, option):
		tsize = typeinfo.typemap.getsize (type)
		precision = 10 ** (tsize-2)
		try:
			val = [int ((round (float (x), tsize) + 180.0) * precision) for x in value.split ("/", 2)]
			assert (len (val) == 2)
			if [x for x in val if x > (360 * precision)]:
				raise ValueError
		except:
			val = [0, 0]
		self.segment.writeSortKey (num, val, tsize)

	def __call__ (self, type, docid, num, name, value, lang, option):
		method = getattr (self, "index" + typeinfo.typemap.getmethod (type))
		method (type, docid, num, name, value, lang, option)



#!/usr/bin/env python

f = open ("wissen/__init__.py", "r")
for line in f:
	if line.startswith ("VERSION"):		
		__VER__ = line [10:].strip ()[1:-1]
		break
f.close ()

import os
import glob
from warnings import warn

try:
	from setuptools import setup, Extension
except ImportError:	
	from distutils.core import setup, Extension
from distutils.sysconfig import get_python_lib		
import platform
import sys

if sys.argv[-1] == 'publish':
	os.system('python setup.py sdist upload') # bdist_wininst --target-version=2.7
	sys.exit()

modules = []

dirv = "%d.%d" % sys.version_info [:2]
if dirv == "3.3":
	dirv = "3.4"
osbit, _dummy = platform.architecture()
dira = osbit == "32bit" and "x86" or "x64"

if os.name == "nt":
	include_dirs = [
		"wissen/win32inclib/zlib/%s" % dira, 
		"wissen/win32inclib/pthread2/%s/%s" % (dirv, dira)
	]
	library_dirs = include_dirs
	libraries = ["zlib", "pthreadVC2"]
		
else:
	include_dirs = ["/usr/include/x86_64-linux-gnu", "/usr/include"]
	library_dirs = ["/usr/lib/x86_64-linux-gnu", "/usr/lib"]
	libraries = ["z", "pthread"]

module = Extension(
	'wissen._wissen',
	sources = [
		'wissen/extension/wissen/core.c', 
		'wissen/extension/wissen/analyzer/analyzer.c',		
		'wissen/extension/wissen/analyzer/stopword.c', 
		'wissen/extension/wissen/analyzer/endword.c', 
		'wissen/extension/wissen/analyzer/formalizer.c', 		
		'wissen/extension/wissen/analyzer/stem.c',
		'wissen/extension/wissen/analyzer/stem_de.c',
		'wissen/extension/wissen/analyzer/stem_fr.c',
		'wissen/extension/wissen/analyzer/stem_it.c',
		'wissen/extension/wissen/analyzer/stem_fi.c',
		'wissen/extension/wissen/analyzer/stem_es.c',
		'wissen/extension/wissen/analyzer/stem_hu.c',
		'wissen/extension/wissen/analyzer/stem_pt.c',
		'wissen/extension/wissen/analyzer/stem_sv.c',
		'wissen/extension/wissen/analyzer/stem_ar.c',
		'wissen/extension/wissen/analyzer/removeaccents.c',
		'wissen/extension/wissen/index/sort.c', 
		'wissen/extension/wissen/index/osutil.c', 
		'wissen/extension/wissen/index/search.c', 
		'wissen/extension/wissen/index/compfunc.c',
		'wissen/extension/wissen/index/heapsort.c', 
		'wissen/extension/wissen/index/termhashtable.c', 
		'wissen/extension/wissen/index/generichash.c',
		'wissen/extension/wissen/index/bfile.c', 
		'wissen/extension/wissen/index/zip.c',
		'wissen/extension/wissen/index/mempool.c',
		'wissen/extension/wissen/index/ibucket.c',
		'wissen/extension/wissen/index/util.c',
		'wissen/extension/wissen/mod_analyzer.c',		
		'wissen/extension/wissen/mod_util.c', 
		'wissen/extension/wissen/mod_int.c', 
		'wissen/extension/wissen/mod_bits.c', 
		'wissen/extension/wissen/mod_mergeinfo.c', 
		'wissen/extension/wissen/mod_document.c',
		'wissen/extension/wissen/mod_posting.c', 
		'wissen/extension/wissen/mod_termtable.c',
		'wissen/extension/wissen/mod_terminfo.c',
		'wissen/extension/wissen/mod_sortmap.c',   
		'wissen/extension/wissen/mod_memorypool.c',		
		'wissen/extension/wissen/mod_selector.c',		
		'wissen/extension/wissen/mod_classifier.c',
		'wissen/extension/wissen/mod_dbint.c',
		'wissen/extension/wissen/mod_calculator.c',
		'wissen/extension/wissen/mod_binfile.c',		
		'wissen/extension/wissen/mod_sgmlparser.c',
		'wissen/extension/wissen/mod_compute.c',
	],
	include_dirs = include_dirs,
	library_dirs = library_dirs,
	libraries = libraries
)
modules.append (module)

packages = [
	'wissen',
	'wissen.analyzers',
	'wissen.analyzers.util',
	'wissen.searcher',	
	'wissen.searcher.segment',
	'wissen.classifier',	
	'wissen.classifier.classifiers',
	'wissen.classifier.segment',	
	'wissen.cluster',
	'wissen.util',
	'wissen.lib'
]

package_dir = {
	'wissen': 'wissen'
}
   
data_files = [
	"README.md",
	"extension/wissen/*.*",
	"extension/wissen/analyzer/*.*",
	"extension/wissen/index/*.*",	
	"extension/wissen/win32/*.*",	
	"win32inclib/pthread2/msvcr90.dll",
	"win32inclib/pthread2/2.7/x64/*.*",
	"win32inclib/pthread2/2.7/x86/*.*",
	"win32inclib/pthread2/3.4/x64/*.*",
	"win32inclib/pthread2/3.4/x86/*.*",
	"win32inclib/zlib/x64/*.*",
	"win32inclib/zlib/x86/*.*"
]

if os.name == "nt" and ("build" in sys.argv or "install" in sys.argv):
	import shutil		
	if os.path.isfile ("wissen/pthreadVC2.dll"):
		os.remove ("wissen/pthreadVC2.dll")
	shutil.copy ("wissen/win32inclib/pthread2/%s/%s/pthreadVC2.dll" % (dirv, dira), "wissen/pthreadVC2.dll")
	
	if os.path.isfile ("wissen/msvcr90.dll"):
		os.remove ("wissen/msvcr90.dll")
	shutil.copy ("wissen/win32inclib/pthread2/%s/msvcr90.dll" % dirv, "wissen/msvcr90.dll")
	
	data_files.extend (["pthreadVC2.dll", "msvcr90.dll"])
	
package_data = {
	"wissen": data_files
}

classifiers = [
  'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
  'Development Status :: 2 - Pre-Alpha',
  'Environment :: Console',	
	'Topic :: Software Development :: Libraries :: Python Modules',
	'Intended Audience :: Developers',
	'Intended Audience :: Science/Research',
	'Programming Language :: Python',
	'Programming Language :: Python :: 2.7',
	'Programming Language :: Python :: 3',
	'Topic :: Text Processing :: Indexing'	
]


setup (
	name = 'wissen',
	version = __VER__,
	author = 'Hans Roh',
	description='Wissen Full-Text Search & Classification Engine',
	author_email = 'hansroh@gmail.com',
	url = 'http://osp.skitai.com',
	packages=packages,
	package_dir=package_dir,
	package_data = package_data,
	license='BSD',
	platforms = ["posix", "nt"],
	download_url = "https://pypi.python.org/pypi/wissen",
	classifiers=classifiers,
	ext_modules = modules
)

